dnl -*- sh -*-
dnl
dnl LSC_FIND_MOTIF
dnl --------------
dnl
dnl Find Motif libraries and headers
dnl Put Motif include directory in motif_includes,
dnl Put Motif library directory in motif_libraries,
dnl and add appropriate flags to X_CFLAGS and X_LIBS.
dnl
dnl

AC_DEFUN([OSL_FIND_MOTIF],[

# Make sure we have X libraries / includes
AC_REQUIRE([AC_PATH_XTRA])
motif_includes=""
osl_found_motif_includes="0"
motif_libraries=""
osl_found_motif_libraries="0"

AC_ARG_WITH(motif,
[  --with-motif=DIR          Motif files are in DIR/include and DIR/lib],
motif_includes="$withval/include" ; motif_libraries="$withval/lib")
AC_ARG_WITH(motif-includes,
[  --with-motif-includes=DIR Motif include files are in DIR],
motif_includes="$withval")
AC_ARG_WITH(motif-libs,
[  --with-motif-libs=DIR     Motif libraries are in DIR],
motif_libraries="$withval")
AC_MSG_CHECKING(for Motif)

#
# Do the work
#
OSL_FIND_MOTIF_INCLUDES
OSL_FIND_MOTIF_LIBRARIES


#
# Do the results
#
osl_motif_includes_result=""
if test "$osl_found_motif_includes" = "0"; then
    motif_includes="(none)"
    osl_motif_includes_result="(none)"
else
    if test "$motif_includes" = ""; then
	osl_motif_includes_result="in default path"
    else
	osl_motif_includes_result="$motif_includes"
    fi
fi

osl_motif_libraries_result=""
if test "$osl_found_motif_libraries" = "0"; then
    motif_libraries="(none)"
    osl_motif_libraries_result="(none)"
else
    if test "$motif_libraries" = ""; then
	osl_motif_libraries_result="in default path"
    else
	osl_motif_libraries_result="$motif_libraries"
    fi
fi

AC_MSG_RESULT(
  [libraries $osl_motif_libraries_result, headers $osl_motif_includes_result])

unset osl_motif_libraries_result osl_motif_includes_result
unset osl_found_motif_includes osl_found_motif_libraries
])dnl


#
#
# OSL_FIND_MOTIF_INCLUDES
#
#
AC_DEFUN([OSL_FIND_MOTIF_INCLUDES],[

LIBS_save="$LIBS"
INCLUDES_save="$INCLUDES"
CPPFLAGS_save="$CPPFLAGS"
LDFLAGS_save="$LDFLAGS"

LIBS="$X_PRE_LIBS -lXm -lXt -lX11 $X_EXTRA_LIBS $LIBS"
INCLUDES="$X_CFLAGS $INCLUDES"
CPPFLAGS="$X_CFLAGS $CPPFLAGS"
LDFLAGS="$X_LIBS $LDFLAGS"

#
# Try specified location
#
if test ! "$motif_includes" = ""; then
    CPPFLAGS="$CPPFLAGS -I$motif_includes"
    AC_TRY_COMPILE([#include <Xm/Xm.h>],[int a;],
	[osl_found_motif_includes="1"],
	[osl_found_motif_includes="0"])
    CPPFLAGS="$CPPFLAGS_save"
fi

#
# Try default locations
#
if test "$osl_found_motif_includes" = "0"; then
    AC_TRY_COMPILE([#include <Xm/Xm.h>],[int a;],
	[osl_found_motif_includes="1" ; motif_includes=""],
	[osl_found_motif_includes="0"])
fi

# Xm/Xm.h is not in the standard search path.
# Locate it and put its directory in `motif_includes'
#
# /usr/include/Motif* are used on HP-UX (Motif).
# /usr/include/X11* are used on HP-UX (X and Athena).
# /usr/dt is used on Solaris (Motif).
# /usr/openwin is used on Solaris (X and Athena).
# Other directories are just guesses.
for dir in "$x_includes" "${prefix}/include" /usr/include /usr/local/include \
    /usr/include/Motif2.1 /usr/include/Motif2.0 /usr/include/Motif1.2  \
    /usr/include/X11 /usr/include/X11R6 /usr/include/X11R5 \
    /usr/X11/include /usr/X11R6/include /usr/X11R5/include \
    /usr/dt/include /usr/openwin/include \
    /usr/dt/*/include /opt/*/include /usr/include/Motif* \
    "${prefix}"/*/include /usr/*/include /usr/local/*/include \
    "${prefix}"/include/* /usr/include/* /usr/local/include/*; do

    if test -f "$dir/Xm/Xm.h"; then
	CPPFLAGS="$CPPFLAGS -I$dir"
	AC_TRY_COMPILE([#include <Xm/Xm.h>],[int a;],
	    [osl_found_motif_includes="1"],
	    [osl_found_motif_includes="0"])
	CPPFLAGS="$CPPFLAGS_save"

	if test "$osl_found_motif_includes" = "1"; then
	    motif_includes="$dir"
	    break
	fi
    fi
done

LIBS="$LIBS_save"
INCLUDES="$INCLUDES_save"
CPPFLAGS="$CPPFLAGS_save"
LDFLAGS="$LDFLAGS_save"

#
# Add Motif definitions to X flags
#
if test "$motif_includes" != "" && test "$motif_includes" != "$x_includes" && test "$motif_includes" != "no" ;then
    X_CFLAGS="-I$motif_includes $X_CFLAGS"
fi

])dnl


#
#
# OSL_FIND_MOTIF_LIBRARIES
#
#
AC_DEFUN([OSL_FIND_MOTIF_LIBRARIES],[

LIBS_save="$LIBS"
INCLUDES_save="$INCLUDES"
CPPFLAGS_save="$CPPFLAGS"
LDFLAGS_save="$LDFLAGS"

LIBS="$X_PRE_LIBS -lXm -lXt -lX11 $X_EXTRA_LIBS $LIBS"
INCLUDES="$X_CFLAGS $INCLUDES"
CPPFLAGS="$X_CFLAGS $CPPFLAGS"
LDFLAGS="$X_LIBS $LDFLAGS"


#
# Try specified location
#
if test ! "$motif_libraries" = ""; then
    LDFLAGS="$LDFLAGS -L$motif_libraries"
    AC_TRY_LINK([#include <Xm/Xm.h>],[XtToolkitInitialize();],
	[osl_found_motif_libraries="1"],
	[osl_found_motif_libraries="0"])
    LDFLAGS="$LDFLAGS_save"
fi

#
# Try default locations
#
if test "$osl_found_motif_libraries" = "0"; then
    AC_TRY_LINK([#include <Xm/Xm.h>],[XtToolkitInitialize();],
	[osl_found_motif_libraries="1"],
	[osl_found_motif_libraries="0"])
fi


# libXm.a is not in the standard search path.
# Locate it and put its directory in `motif_libraries'
#
# /usr/lib/Motif* are used on HP-UX (Motif).
# /usr/lib/X11* are used on HP-UX (X and Athena).
# /usr/dt is used on Solaris (Motif).
# /usr/lesstif is used on Linux (Lesstif).
# /usr/openwin is used on Solaris (X and Athena).
# Other directories are just guesses.
for dir in "$x_libraries" "${prefix}/lib" /usr/lib /usr/local/lib \
           /usr/lib/Motif2.1 /usr/lib/Motif2.0 /usr/lib/Motif1.2 \
           /usr/lib/X11 /usr/lib/X11R6 /usr/lib/X11R5 \
           /usr/X11/lib /usr/X11R6/lib /usr/X11R5/lib \
           /usr/dt/lib /usr/openwin/lib \
           /usr/dt/*/lib /opt/*/lib /usr/lib/Motif* \
           /usr/lesstif*/lib /usr/lib/Lesstif* \
           "${prefix}"/*/lib /usr/*/lib /usr/local/*/lib \
           "${prefix}"/lib/* /usr/lib/* /usr/local/lib/*; do

    if test -d "$dir" && test "`ls $dir/libXm.* 2> /dev/null`" != ""; then

	LDFLAGS="$LDFLAGS -L$dir"
	AC_TRY_LINK([#include <Xm/Xm.h>],[XtToolkitInitialize();],
	    [osl_found_motif_libraries="1"],
	    [osl_found_motif_libraries="0"])
	LDFLAGS="$LDFLAGS_save"

	if test "$osl_found_motif_libraries" = "1"; then
	    motif_libraries="$dir"
	    break
	fi
    fi
done

LIBS="$LIBS_save"
INCLUDES="$INCLUDES_save"
CPPFLAGS="$CPPFLAGS_save"
LDFLAGS="$LDFLAGS_save"


if test "$motif_libraries" != "" && test "$motif_libraries" != "$x_libraries" && test "$motif_libraries" != "no"; then
    case "$X_LIBS" in
	*-R\ *) X_LIBS="-L$motif_libraries -R $motif_libraries $X_LIBS";;
	*-R*)   X_LIBS="-L$motif_libraries -R$motif_libraries $X_LIBS";;
	*)      X_LIBS="-L$motif_libraries $X_LIBS";;
    esac
fi

])dnl
