/*
 *	XMTV
 *
 * Copyright (c) 2001-2002 The Trustees of Indiana University.  
 *                         All rights reserved.
 * Copyright (c) 1998-2001 University of Notre Dame. 
 *                         All rights reserved.
 * Copyright (c) 1994-1998 The Ohio State University.  
 *                         All rights reserved.
 * 
 * This file is part of the XMTV software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * XMTV source distribution.
 *
 *	$Id: xmtv.c,v 1.6 2002/12/11 17:38:39 brbarret Exp $
 * 
 *	Function:	- LAM graphics server
 *			- Motif
 */

#define _NO_PROTO

#include <Xm/MessageB.h>
#include <Xm/Form.h>
#include <X11/cursorfont.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

#include <events.h>
#include <kio.h>
#include <net.h>
#include <portable.h>
#include <priority.h>
#include <terror.h>
#include <tvreq.h>
#include <typical.h>
#include <xmtv.h>

/*
 * global functions
 */
void			xmtv_bail();
void			xmtv_setsize();
void			xmtv_setminmax();
Widget			xmtv_wmshell();

/*
 * external functions
 */
extern Widget		xmtv_ctl_build();
extern Widget		xmtv_view_build();
extern void		xmtv_view_clear();
extern void		xmtv_view_rect();
extern void		xmtv_view_snake();
extern void		xmtv_view_text();
extern void		xmtv_view_state();

/*
 * local functions
 */
static int		reqfront();
static void		reqback();
static void		help();

/*
 * global variables
 */
AppData			app_res;	/* application resources */

/*
 * external variables
 */
extern struct kio_t	_kio;


/*
 * local variables
 */
static struct nmsg	reqhead;	/* network message desc. */
static char		reqbuf[MAXNMSGLEN];
					/* network message body */
static Widget		toplevel;	/* toplevel window */

static int		hinitmin = -1;	/* initial drawable min height */
static int		winitmin = -1;	/* initial drawable min width */
static int		hinitmax = -1;	/* initial drawable max height */
static int		winitmax = -1;	/* initial drawable max width */
static int		hsetup = -1;	/* height setup overhead */
static int		wsetup = -1;	/* width setup overhead */

static XrmOptionDescRec	cmdoptions[] = {
	{"-h", "*help", XrmoptionNoArg, "TRUE"},
	{"-x", "*xsize", XrmoptionSepArg, NULL},
	{"-y", "*ysize", XrmoptionSepArg, NULL},
};

static XtResource	resources[] = {
	{ XtNhelp, XtCHelp, XmRBoolean, sizeof(Boolean),
		XtOffset(AppDataPtr, ap_help),
		XmRImmediate, (caddr_t) False },
	{ XtNxsize, XtCXsize, XmRInt, sizeof(int),
		XtOffset(AppDataPtr, ap_xsize),
		XmRImmediate, (caddr_t) XMTV_DEFXSIZE },
	{ XtNysize, XtCYsize, XmRInt, sizeof(int),
		XtOffset(AppDataPtr, ap_ysize),
		XmRImmediate, (caddr_t) XMTV_DEFYSIZE },
};

/*
 * fallback resources
 */
static String fallbacks[] = {
	"*Title: XMTV",
	"*IconName: XMTV",
	"*multiClickTime: 500",
	"*fontList: -*-helvetica-bold-r-normal--*-120-*-*-*-*-*-*",
	"*background: gray",
	"*ctl_bar.background: slateblue",
	"*ctl_bar.foreground: white",
	"*view_draw.background: black",
	"*view_draw.foreground: gray",
	NULL
};

/*
 *	main
 */
main(argc, argv)

int			argc;
char			*argv[];

{
	Widget		topmgmt_w;
	Widget		w1, w2;
	XtAppContext	app;
	int		lamfd;

	toplevel = XtVaAppInitialize(&app, "XMTV",
		cmdoptions, XtNumber(cmdoptions),
		&argc, argv, fallbacks, NULL);

	XtGetApplicationResources(toplevel,
		&app_res, resources, XtNumber(resources), NULL, 0);

	if (app_res.ap_help) {
		help();
		exit(0);
	}

	if ((app_res.ap_xsize <= 0) || (app_res.ap_ysize <= 0)) {
		fprintf(stderr, "xmtv: invalid screen size\n");
		exit(1);
	}

	if (app_res.ap_xsize < XMTV_MINXSIZE) app_res.ap_xsize = XMTV_MINXSIZE;
	if (app_res.ap_ysize < XMTV_MINYSIZE) app_res.ap_ysize = XMTV_MINYSIZE;
/*
 * Become a Trollius process.
 */
	if (kinit(PRCMD)) {
		terror("xmtv (kinit)");
		exit(errno);
	}

	_kio.ki_rtf |= RTF_SYSGEN;

	if (lpattach("xmtv")) lamfail("xmtv (lpattach)");

	_lam_atexit(xmtv_bail);

	topmgmt_w = XtVaCreateWidget("top_mgr",
		xmFormWidgetClass, toplevel,
		XmNmarginHeight, 2,
		XmNmarginWidth, 2,
		NULL);
/*
 * Create main application view.
 */
	w1 = xmtv_ctl_build(topmgmt_w);
	XtVaSetValues(w1,
		XmNtopAttachment, XmATTACH_FORM,
		XmNleftAttachment, XmATTACH_FORM,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);

	w2 = xmtv_view_build(topmgmt_w);
	XtVaSetValues(w2,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, w1,
		XmNbottomAttachment, XmATTACH_FORM,
		XmNleftAttachment, XmATTACH_FORM,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);

	XtManageChild(topmgmt_w);
	XSetIOErrorHandler((XIOErrorHandler) xmtv_bail);
/*
 * Prime a request and add file desc. to event loop.
 */
	lamfd = reqfront();
	if (lamfd < 0) lamfail("xmtv (bfrecvfront)");

	XtAppAddInput(app, lamfd,
			(XtPointer) XtInputReadMask, reqback, NULL);

	XtRealizeWidget(toplevel);

	xmtv_setminmax(winitmin, hinitmin, winitmax, hinitmax);

	XtAppMainLoop(app);
	return(0);
}

/*
 *	xmtv_bail
 *
 *	Function:	- exits gracefully with 0 status
 */
void
xmtv_bail()

{
/*
 *	lam_kexit(0);
 */
}

/*
 *        help
 *
 *        Function:     - prints helpful information on this command
 */
static void
help()

{
	printf("\nSynopsis:\txmtv [<options>]\n");
	printf("\nDescription:\tX Windows LAM graphics daemon\n");
	printf("\nOptions:\t-h\t\tPrint this help message.\n");
	printf("\t\t-x <xsize>\tSet width to <xsize> pixels.\n");
	printf("\t\t-y <ysize>\tSet height to <ysize> pixels.\n");
}

/*
 *	xmtv_wmshell
 *
 *	Function:	- returns the first WMShell ancestor
 *	Accepts:	- widget
 *	Returns:	- window manager shell
 */
Widget
xmtv_wmshell(w)

Widget			w;

{
	while ((w) && ( ! XtIsWMShell(w))) {
		w = XtParent(w);
	}

	return(w);
}

/*
 *	xmtv_setminmax
 *
 *	Function:	- set min/max toplevel shell size
 *	Accepts:	- min width
 *			- min height
 *			- max width
 *			- max height
 */
void
xmtv_setminmax(wmin, hmin, wmax, hmax)

int			wmin;
int			hmin;
int			wmax;
int			hmax;

{
	Dimension	height;
	Dimension	width;
/*
 * The first time around, just store the drawable sizes.
 */
	if (winitmin < 0) {
		winitmin = wmin;
		hinitmin = hmin;
		winitmax = wmax;
		hinitmax = hmax;
	}
	else {
/*
 * The second time around, compute the width/height window overhead.
 */
		if (wsetup < 0) {
			XtVaGetValues(toplevel, XmNwidth, &width,
					XmNheight, &height, NULL);

			wsetup = width - wmax;
			hsetup = height - hmax;
		}
/*
 * Set the size boundaries.
 */
		XtVaSetValues(toplevel,
			XmNminWidth, wsetup + wmin,
			XmNminHeight, hsetup + hmin,
			XmNmaxWidth, wsetup + wmax,
			XmNmaxHeight, hsetup + hmax,
			NULL);
	}
}

/*
 *	xmtv_setsize
 *
 *	Function:	- set toplevel shell size
 *	Accepts:	- width
 *			- height
 */
void
xmtv_setsize(width, height)

int			width;
int			height;

{
	XResizeWindow(XtDisplay(toplevel), XtWindow(toplevel),
		(unsigned) wsetup + width, (unsigned) hsetup + height);
}

/*
 *	xmtv_seticon
 *
 *	Function:	- set the tool icon
 *	Accepts:	- icon pixmap
 */
void
xmtv_seticon(iconpix)

Pixmap			iconpix;

{
	Window		win;
	Window		root;
	Display		*disp;
	int		x, y;
	unsigned int	width, height, bwidth, depth;

	disp = XtDisplay(toplevel);
	XtVaGetValues(toplevel, XmNiconWindow, &win, NULL);

	if (! win) {
		if (! XGetGeometry(disp, iconpix, &root, &x, &y,
				&width, &height, &bwidth, &depth) ||
			!(win = XCreateSimpleWindow(disp, root, 0, 0,
				width, height, (unsigned int) 0,
				CopyFromParent, CopyFromParent))) {

			XtVaSetValues(toplevel, XmNiconPixmap, iconpix, NULL);
			return;
		}

		XtVaSetValues(toplevel, XmNiconWindow, win, NULL);
	}

	XSetWindowBackgroundPixmap(disp, win, iconpix);
	XClearWindow(disp, win);
}

/*
 *	reqfront
 *
 *	Function:	- setup a receive request
 *	Returns:	- socket or ERROR
 */
static int
reqfront()

{
	reqhead.nh_event = EVMTVD;
	reqhead.nh_type = 0;
	reqhead.nh_flags = 0;
	reqhead.nh_length = sizeof(reqbuf);
	reqhead.nh_msg = reqbuf;

	return(bfrecvfront(&reqhead));
}

/*
 *	reqback
 *
 *	Function:	- finish the receive request
 *			- handle the request
 *			- setup a new receive request
 */
static void
reqback()

{
	struct tvreq	*req;
	struct tvreply	*reply;
	int		ret;

	ret = bfrecvback(&reqhead);
/*
 * If interrupted, retry.
 */
	if ( ((ret > 0) && ((ret & _kio.ki_sigretry) == ret)) ||
				((ret < 0) && (errno == ETIMEDOUT))) {
		reqfront();
		return;
	}

	if (ret) {
		return;
	}
/*
 * Handle received request.
 */
	req = (struct tvreq *) reqhead.nh_data;

	switch(req->tvq_req) {

	case TVQCLR:
		xmtv_view_clear();
		break;

	case TVQRECT:
		xmtv_view_rect(req->tvq_x1, req->tvq_y1, req->tvq_xlen,
				req->tvq_ylen, reqhead.nh_msg);
		break;

	case TVQSNAKE:
		xmtv_view_snake(req->tvq_x1, req->tvq_len, reqhead.nh_msg);
		break;

	case TVQTEXT:
		xmtv_view_text(req->tvq_x1, req->tvq_y1,
				req->tvq_colour, reqhead.nh_msg);
		break;

	case TVQSTATE:
		reply = (struct tvreply *) reqhead.nh_data;

		reqhead.nh_node = req->tvq_src_node;
		reqhead.nh_event = req->tvq_src_event;
		reqhead.nh_type = 0;
		reqhead.nh_flags = 0;
		reqhead.nh_length = 0;
		reqhead.nh_msg = 0;

		xmtv_view_state(&reply->tvr_xlen, &reply->tvr_ylen,
				&reply->tvr_ncolour, &reply->tvr_xtext,
				&reply->tvr_ytext);

		reply->tvr_ret = 0;

		if (nsend(&reqhead)) {
			return;
		}
		break;

	default:
		break;
	}

	reqfront();
}
